/***************************************************************************
 *            FSK.c
 *
 *  Fr Jun 04 17:54:00 2010
 *  Copyright  2010  Albrecht Lohoefener
 *  albrechtloh@gmx.de
 ****************************************************************************/


/**
* \file
* Diese Datei erzeugt die FSK Modulation und liest die zu sendenden Daten aus einem Ringbuffer aus. Die Erzeugung der verschiedenen FSK Frequenzen uebernimmt ein PWM Timer im CTC Modus.
*/

#include <avr/io.h>

#include "FSK.h"
#include "PWM.h"
#include "LED.h"
#include "RingBuffer.h"
#include "Timer.h"

/**
 * Dauer eines Bits in ms/2 ("2" siehe FSKSendStart() -> TimerSet())
 */
#define FSKBitDuration 2

/**
 * Dauer eines Sync Bits in ms/2 (FSKSyncBitDuration = FSKBitDuration * 3/2)
 */
#define FSKSyncBitDuration 3

/**
 *  Dauer eines Break Bits in ms/2 (FSKBreakBitDuration = FSKBitDuration)
 */
#define FSKBreakBitDuration 2

/**
 * Frequenz fuer 0 in Hz
 */
#define FSKFreq0 1000

/**
 * Frequenz fuer 1 in Hz
 */
#define FSKFreq1 2000


// Lokale Prototypen
 /**
  * \brief  Holt ein Byte aus dem RingBuffer
  *         Wurde diese Funktion 8mal aufgerufen, so ist das gesamte Byte gesendet
  *
  * \param 	keine
  * \return	keine
  *
  */
void FSKSendDataBit(void);

 /**
  * \brief  Sendet eine Pause
  *
  * \param 	keine
  * \return	keine
  *
  */
void FSKSendBreakBit(void);

 /**
  * \brief  Sendet das SyncBit
  *
  * \param 	keine
  * \return	keine
  *
  */
void FSKSendSyncBit(void);


// Globale Variablen
/**
 * Bit Position innerhalb einer Nachricht 
 * 0:SyncBit 1:BreakBit 2-9:DataBits 10:BreakBit
 */
volatile uint8_t FSKMessagePos = 0;

void FSKSendStart(void)
{
	// PWM starten 
	PWMStart(FSKFreq0);

	// Timer zum Dauersenden starten, periodisch aller 2 ms
	TimerSet(FSK,period,2,&FSKSend);
	TimerStart(FSK);
}

void FSKSend(void)
{
	static uint8_t FSKInterruptCounter = 0;

	FSKInterruptCounter ++;

	// Data Bit
	if(FSKMessagePos > 1 && FSKMessagePos < 10)
	{
		if(FSKInterruptCounter == 1)
			FSKSendDataBit();

		if(FSKInterruptCounter > FSKBitDuration)
		{
			// FSKInterruptCounter nullen
			FSKInterruptCounter = 0;

			// Message Position um eins erhoehen
			FSKMessagePos++;

			// Interrupt Routine neu rufen, damit keine Pause entsteht
			FSKSend();
		}
	}

	// Break Bit
	if(FSKMessagePos == 1 || FSKMessagePos == 10)
	{
		if(FSKInterruptCounter == 1)
			FSKSendBreakBit();

		if(FSKInterruptCounter > FSKBreakBitDuration)
		{
			FSKInterruptCounter = 0;

			// Message Position um eins erhoehen
			if(FSKMessagePos == 10)
				FSKMessagePos = 0;
			else
				FSKMessagePos++;

			// Interrupt Routine neu rufen damit keine Pause entsteht
			FSKSend();
		}
	}

	// Sync Bit
	if(FSKMessagePos == 0)
	{
		if(FSKInterruptCounter == 1)
			FSKSendSyncBit();

		if(FSKInterruptCounter > FSKSyncBitDuration)
		{
			// FSKInterruptCounter nullen
			FSKInterruptCounter = 0;
			// Message Position um eins erhoehen
			FSKMessagePos++;
			
			// Interrupt Routine neu rufen damit keine Pause entsteht
			FSKSend();
		}
		
	}
}


// Lokale Funktionen
void FSKSendDataBit(void)
{
	static uint8_t FSKSendDataBitBitCounter = 0;
	static uint8_t FSKSendDataBitByte = 0;
	static uint8_t FSKSendDataBitBit = 0;

	// Ein neues Byte aus dem RingBuffer holen
	if(FSKSendDataBitBitCounter == 0)
	{
		FSKSendDataBitByte = RingBufferGetValue();
	}

	// Bit holen
	FSKSendDataBitBit = FSKSendDataBitByte & 0x1;
	// Byte um eins nach rechts verschieben
	FSKSendDataBitByte = FSKSendDataBitByte >> 1;
	
	// DataBit senden
	if(FSKSendDataBitBit)
		PWMSetFrequency(FSKFreq1);
	else
		PWMSetFrequency(FSKFreq0);

	// Bit Counter erhoehen
	if(FSKSendDataBitBitCounter >= 7)
		FSKSendDataBitBitCounter = 0;
	else
		FSKSendDataBitBitCounter++;
}

void FSKSendBreakBit(void)
{
	// BreakBit mittels Freq0 senden
	PWMSetFrequency(FSKFreq0);
}

void FSKSendSyncBit(void)
{
	// SyncBit mittels Freq1 senden
	PWMSetFrequency(FSKFreq1);
}
