/***************************************************************************
 *            HeartRate.c
 *
 *  Sun Jun 06 13:41:00 2010
 *  Copyright  2010  Albrecht Lohoefener
 *  albrechtloh@gmx.de
 ****************************************************************************/

/**
* \file
* Initialisiert die Detektion des Herzpulses.
* Berechnet den Herztakt. Algorythmus zur Berechnung: Der Empfaenger sendet Pulse aus. Bei jedem Puls wird HeartRateInterrupt() gerufen und ein Zähler hochgezaehlt. Die Funktion HeartRateCalculate() wird aller 3 sec gerufen und liest den Zaehler aus. Dieser Zaehler ist ein Mass fuer den Herzfrequenz. Diese Herzfrequenz wird danach in einen Ringpuffer geschrieben.
*/

#include <avr/io.h>

#include "HeartRate.h"
#include "RingBuffer.h"
#include "Interrupt.h"
#include "LED.h"
#include "Timer.h"

/**
*	Zeitbasis fuer Zeitmessung zwischen zwei Pulse in ms
*/
#define HEARTRATETIMEBASE 2

// Definiert den Heartratepuffer
HeartRateValueBuffer_t HeartRateValueBuffer;

void HeartRateInit(void)
{
	// Exteren Interrupt init
	InterruptHeartRateInit();

	// Zeitzeahler ruecksetzen
	HeartRateTimeCounter = 0;

	// Heartratepuffer init
	HeartRateValueBuffer.WritePointer = 0;

	// LEDTimer init
	TimerSet(HeartRateLED,single,100,&LEDoff);

	// LED blinken, d.h. Programm gestartet, noch kein Empfang der Herzfrequenz
	TimerSet(LED,period,50,&LEDtoggle);
	TimerStart(LED);

	// Starten des Timers zur Berechnung der Herzfrequenz
	TimerSet(HeartRateCalc,period,2000,&HeartRateCalculate);
	TimerStart(HeartRateCalc);

	// Starten des Timers zum Messen der Zeit zwischen zwei Pulse
	TimerSet(HeartRateTime,period,HEARTRATETIMEBASE,&HeartRateIncTime);
	TimerStart(HeartRateTime);
}

void HeartRateCalculate(void)
{
	uint8_t HeartRate = 0;
	uint16_t HeartRateTime = 0;
	uint8_t i = 0;
	uint8_t isWorse = 0;

	// Wenn 60 sec kein Impulse erkannt wurde, dann LED schnell blinken
	if(HeartRateTimeCounter >= ((uint32_t)60 * (uint32_t)1000 / (uint32_t)HEARTRATETIMEBASE))
	{
		// LED blinken
		TimerStart(LED);

		// Zeitzaehler zurueck setzen
		HeartRateTimeCounter = 0;

		// 0 in Puffer schreiben um zu signalisieren, dass keine neuen Werte dazu gekommen sind
		HeartRateValueBuffer.Time[0] = 0;
	}

	// Zeiten aus Heatratebuffer aufaddieren
	for(i=0;i<HEARTRATEBUFFERSIZE;i++)
	{
		// Gesamten Puffer wegschmeissen, wenn ein Wert 0 enthaelt
		if(HeartRateValueBuffer.Time[i] == 0 ) isWorse = 1;

		HeartRateTime += HeartRateValueBuffer.Time[i];
	}

	// Durchschnitt bilden
	HeartRateTime /= HEARTRATEBUFFERSIZE;

	// Herzfrequenz berechnen
	HeartRate = (uint32_t)1000 * (uint32_t)60 / ((uint32_t)HeartRateTime * (uint32_t)HEARTRATETIMEBASE);

		// Herzfrequenz in Ringpuffer speichern, wenn keine 0 vorhanden war
	if(isWorse == 0 )
		RingBufferSetValue(HeartRate);

}

void HeartRateInterrupt(void)
{
	uint16_t* pTime = &HeartRateValueBuffer.Time[0];

	/*static uint8_t testvar=0;

	if(testvar == 0) HeartRateTimeCounter = 370;
	if(testvar == 1) HeartRateTimeCounter = 379;
	if(testvar == 2) HeartRateTimeCounter = 352;
	if(testvar == 3) HeartRateTimeCounter = 333;
	testvar++;
	if(testvar >=4) testvar=0;*/

	// Pruefen ob Heartfrequenz zwischen 40 und 230 liegt
	if((HeartRateTimeCounter <= (uint32_t)1000*(uint32_t) 60 / ((uint32_t)HEARTRATETIMEBASE * (uint32_t)40)) &&
		(HeartRateTimeCounter >= (uint32_t)1000*(uint32_t) 60 / ((uint32_t)HEARTRATETIMEBASE * (uint32_t)230)) )
	{
		// Zeit aus HeartRateTimeCounter in Puffer speichern
		pTime[HeartRateValueBuffer.WritePointer] = HeartRateTimeCounter;	
	}
	else
	{	
		// Eine 0 in Puffer speichern, zeigt an, dass dieser Wert ungueltig ist
		pTime[HeartRateValueBuffer.WritePointer] = 0;
	}

	// Auf naechtes Element zeigen
	HeartRateValueBuffer.WritePointer++;

	// Puffer UEberlauf, von vorne beginnen
	if(HeartRateValueBuffer.WritePointer >= HEARTRATEBUFFERSIZE)
		HeartRateValueBuffer.WritePointer = 0;

	// Zeitzeahler ruecksetzen
	HeartRateTimeCounter = 0;

	// LED Timer stoppen, damit die LED die Herzfrequenz anzeigen kann
	TimerStop(LED);

	// LED im Takt des Herze Blinken
	LEDon();
	// Wenn Timer abgelaufen ist LED ausschalten
	TimerStart(HeartRateLED);

	RingBufferSetValue(1);
}

void HeartRateIncTime(void)
{
	HeartRateTimeCounter++;
}
