/***************************************************************************
 *            PWM.c
 *
 *  Mo May 31 16:34:02 2010
 *  Copyright  2010  Albrecht Lohoefener
 *  albrechtloh@gmx.de
 ****************************************************************************/

/**
* \file
* Regelt den Zugriff auf den PWM Timer: Initalisierung im CTC Modus, Start, Stop und setzen der Ausgabefrequenz
*/ 

#include <avr/io.h>
#include <stdint.h>

#include "PWM.h"

/**
*	Vorteiler fuer den PWM Timer
*/
#define PRESCALERTIMER0_ATMEGA128 32
#define PRESCALERTIMER1_ATTINY85 16
/**
*	Register Werte fuer Perscaler
*/
#define PRESCALERTIMER0_C_ATMEGA128 ((1 << CS01) | (1 << CS00))
#define PRESCALERTIMER1_C_ATTINY85 ((1 << CS10) | (1 << CS12))

int PWMInit(void)
{
	#ifdef __AVR_ATmega128__
	// PB4 auf Ausgang setzen
	DDRB |= (1 << DDB4);

	// WGM 01 CTC Mode, COM00 (PB3) OC0 toggeln
	TCCR0 = (1 << WGM01) | (1 << COM00);
	#endif

	#ifdef __AVR_ATtiny85__
	// PB1 auf Ausgang setzen
	DDRB |= (1 << DDB1);

	// PWM1A PWM Mode, COM1A0 (PB0) OC1A toggeln
	TCCR1 = (1 << CTC1) | (1 << COM1A0);

	// Wenn TCNT1 0 ist, wird PB1 getoggelt
	OCR1A = 0;
	#endif

	return 0;
}

int PWMStart(int Frequency)
{
	// Frequenz setzen
	PWMSetFrequency(Frequency);

	// Wenn eine Zaehler gesetzt ist, dann PWM einschalten
	#ifdef __AVR_ATmega128__
	if(OCR0)
		TCCR0 |= PRESCALERTIMER0_C_ATMEGA128;
	else
	{
		PWMStop();
		return -1;
	}
	#endif
	#ifdef __AVR_ATtiny85__
	if(OCR1C)
		TCCR1 |= PRESCALERTIMER1_C_ATTINY85;
	else
	{
		PWMStop();
		return -1;
	}
	#endif

	return 0;
}

int PWMStop(void)
{
	// PWM ausschalten, Ruecksetzen des Prescalers
	#ifdef __AVR_ATmega128__
	TCCR0 &= ~ PRESCALERTIMER0_C_ATMEGA128;
	#endif
	#ifdef __AVR_ATtiny85__
	TCCR1 &= ~ PRESCALERTIMER1_C_ATTINY85;
	#endif

	return 0;
}

int PWMSetFrequency(int Frequency)
{
	uint16_t counter_value = 0;

	//Frequency = 1000;

	// Berechnung des Zaehlwertes	
	#ifdef __AVR_ATmega128__
	counter_value = (F_CPU / (uint32_t)(2 * (uint32_t)PRESCALERTIMER0_ATMEGA128 * (uint32_t)Frequency)) - 1;
	#endif
	#ifdef __AVR_ATtiny85__
	counter_value = (F_CPU / (uint32_t)(2 * (uint32_t)PRESCALERTIMER1_ATTINY85 * (uint32_t)Frequency)) - 1;
	#endif

	// UEberpruefen ob Zahlenbreichsueberschreitung vorliegt

	if(counter_value <= 255)
		#ifdef __AVR_ATmega128__
		OCR0 = counter_value;
		#endif
		#ifdef __AVR_ATtiny85__
		{
		//OCR1A = counter_value;
		OCR1C = counter_value;
		}
		#endif
	else
		return -1;

	return 0;
}
