/**
 * de.lohaltec.laufputer: A heart rate meter for Android
 *  
 * <br>Copyright 2010 Albrecht Lohoefener <albrechtloh@gmx.de>
 * Date: 02.10.2010
 * 
 * <p>These classes provide input functions to get raw audio samples from the microphone.
 * Based on Ian Cameron Smith's "AudioAnalyser.java" from http://code.google.com/p/moonblink/wiki/HermitAndroid
 *
 * <p>This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation (see COPYING).
 * 
 * <p>This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package de.lohaltec.laufputer;

import org.hermit.android.io.AudioReader;

public class Audio {
// ******************************************************************** //
// Class Data.
// ******************************************************************** //

// The desired sampling rate for this analyser, in samples/sec.
//private int sampleRate = 22050;
private int sampleRate = 11025;

// Audio input block size, in samples.
private int inputBlockSize = 256;

// If we got a read error, the error code.
private int readError = AudioReader.Listener.ERR_OK;

// Our audio input device.
private final AudioReader audioReader;

// Audio processing object
private final FskDemod audioProcessing;

/**
 * Constructor
 * @param audioProcessing_ Object of the audioProcessing
 */
public Audio(FskDemod audioProcessing_)
{
	// create a new AudioReader instance
	audioReader = new AudioReader();
	
	// save the audio processing object pointer
	audioProcessing = audioProcessing_;
}

//******************************************************************** //
// Control
//******************************************************************** //

/**
 * Start the Audio recording
 */
public void audioStart()
{
	readError = AudioReader.Listener.ERR_OK;
    
    audioReader.startReader(sampleRate, inputBlockSize, new AudioReader.Listener() {
        @Override
        public final void onReadComplete(short[] buffer) {
            receiveAudio(buffer);
        }
        @Override
        public void onReadError(int error) {
            handleError(error);
        }
    });
}

/**
 * Stop the Audio recording
 */
public void audioStop()
{
	audioReader.stopReader();
}

//******************************************************************** //
// Audio Processing.
// ******************************************************************** //

/**
 * Handle audio input.  This is called on the thread of the audio
 * reader.
 * 
 * @param   buffer      Audio data that was just read.
 */
private final void receiveAudio(short[] buffer)
{
	audioProcessing.receiveAudio(buffer,sampleRate,inputBlockSize);
}


/**
 * An error has occurred.  The reader has been terminated.
 * 
 * @param   error       ERR_XXX code describing the error.
 */
private void handleError(int error) {
    synchronized (this) {
        readError = error;
    }
}

//******************************************************************** //
// Configuration.
// ******************************************************************** //

/**
 * Set the sample rate for this instrument.
 * 
 * @param   rate        The desired rate, in samples/sec.
 */
public void setSampleRate(int rate)
{
    sampleRate = rate;
}

/**
 * Set the input block size for this instrument.
 * 
 * @param   size        The desired block size, in samples.  Typical
 *                      values would be 256, 512, or 1024.
 */
public void setBlockSize(int size)
{
    inputBlockSize = size;
}

/**
 * Set the input block size for this instrument.
 * 
 * @return   readError   The error code
 */
public int getError()
{
    return readError;
}

}
